<?php

namespace Bixo\Aml\Http\Controllers;

use Exception;
use Bixo\Aml\Actions\NationalityWorkflow;
use Bixo\Aml\Http\Requests\NationalityWorkflowRequest;
use Bixo\Aml\Http\Resources\NationalityResource;
use Bixo\Aml\Models\Nationality;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class NationalityWorkflowController extends BaseController
{
    /**
     * Action controller function for nationality.
     *
     * @param Model $nationality
     * @param action next action for the nationality.
     *
     * @return Response
     */
    public function __invoke(NationalityWorkflowRequest $request, Nationality $nationality, $trasition)
    {
        try {
            $request = $request->all();
            $nationality = NationalityWorkflow::run($trasition, $nationality, $request);
            $data = new NationalityResource($nationality);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('aml::nationality.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('aml/nationality/' . $nationality->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('aml/nationality/' . $nationality->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the nationality and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $nationality
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $nationality, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $nationality = Nationality::findBySignedId($nationality);
        $transitions = $this->transitions($nationality, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('aml::nationality.name') . ' ' . trans('Approvals'))
            ->view('aml::public.nationality.workflow')
            ->layout('mobile')
            ->data(compact('nationality', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the nationality.
     *
     * @param PublicRequest $request
     * @param string $nationality
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $nationality, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $nationality = Nationality::findBySignedId($nationality);
        $user = user();
        $nationality = NationalityWorkflow::run($transition, $nationality, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($nationality, $user)
    {
        $transitions = [];
        foreach ($nationality->workflow()->transitions($nationality) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.nationality.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'nationality' => $nationality->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $nationality->workflow()->form($value);
            $array['label'] = trans('aml::nationality.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
