<?php

namespace Bixo\Aml\Http\Controllers;

use Exception;
use Bixo\Aml\Actions\SanctionWorkflow;
use Bixo\Aml\Http\Requests\SanctionWorkflowRequest;
use Bixo\Aml\Http\Resources\SanctionResource;
use Bixo\Aml\Models\Sanction;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class SanctionWorkflowController extends BaseController
{
    /**
     * Action controller function for sanction.
     *
     * @param Model $sanction
     * @param action next action for the sanction.
     *
     * @return Response
     */
    public function __invoke(SanctionWorkflowRequest $request, Sanction $sanction, $trasition)
    {
        try {
            $request = $request->all();
            $sanction = SanctionWorkflow::run($trasition, $sanction, $request);
            $data = new SanctionResource($sanction);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('aml::sanction.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('aml/sanction/' . $sanction->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('aml/sanction/' . $sanction->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the sanction and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $sanction
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $sanction, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $sanction = Sanction::findBySignedId($sanction);
        $transitions = $this->transitions($sanction, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('aml::sanction.name') . ' ' . trans('Approvals'))
            ->view('aml::public.sanction.workflow')
            ->layout('mobile')
            ->data(compact('sanction', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the sanction.
     *
     * @param PublicRequest $request
     * @param string $sanction
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $sanction, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $sanction = Sanction::findBySignedId($sanction);
        $user = user();
        $sanction = SanctionWorkflow::run($transition, $sanction, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($sanction, $user)
    {
        $transitions = [];
        foreach ($sanction->workflow()->transitions($sanction) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.sanction.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'sanction' => $sanction->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $sanction->workflow()->form($value);
            $array['label'] = trans('aml::sanction.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
